<?php

/*
 *  PUMA/BibSonomy CSL (ext_bibsonomy_csl) is a TYPO3 extension which
 *  enables users to render publication lists from PUMA or BibSonomy in
 *  various styles.
 *
 *  Copyright notice
 *  (c) 2015 Sebastian Böttger <seboettg@gmail.com>
 *
 *  HothoData GmbH (http://www.academic-puma.de)
 *  Knowledge and Data Engineering Group (University of Kassel)
 *
 *  All rights reserved
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\ExtBibsonomyCsl\ViewHelpers;

use AcademicPuma\RestClient\Model\Document;
use \TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;
use \TYPO3\CMS\Fluid\Core\ViewHelper\TagBuilder;

/**
 *
 *
 * @package ext_bibsonomy_csl
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
class CslDocumentImageViewHelper extends AbstractViewHelper
{

    /**
     * @param Document $document
     * @param string $intraHash
     * @param string $userName
     * @param string $size
     * @param string $width
     *
     * @return TagBuilder
     */
    protected function getTagBuilderImage(Document $document, $intraHash, $userName, $size, $width)
    {

        $fileName = $document->getFileName();

        $action        = "preview";
        $controller    = "Document";
        $extensionName = str_replace('_', '', $this->controllerContext->getRequest()->getControllerExtensionKey());
        $pluginName    = $this->controllerContext->getRequest()->getPluginName();

        $arguments = ["intraHash" => $intraHash, "fileName" => $fileName, "userName" => $userName, "preview" => $size];

        $uriBuilder = $this->controllerContext->getUriBuilder();
        $uriBuilder->reset();
        $uriBuilder->setNoCache(true);
        $src = $uriBuilder->uriFor($action, $arguments, $controller, $extensionName, $pluginName);

        $img = new TagBuilder('img');
        $img->addAttribute('src', $src);
        $img->addAttribute('alt', $fileName);
        $img->addAttribute('style', 'width: ' . $width . 'px;');
        $img->forceClosingTag(true);

        return $img;
    }

    /**
     * @param Document $document
     * @param string $intraHash
     * @param string $userName
     * @param string $size
     * @param string $width
     *
     * @return string
     */
    public function render(Document $document, $intraHash, $userName, $size = "LARGE", $width = "57")
    {

        $img = $this->getTagBuilderImage($document, $intraHash, $userName, $size, $width);

        return $img->render();
    }
}
