<?php

/*
 *  PUMA/BibSonomy CSL (ext_bibsonomy_csl) is a TYPO3 extension which
 *  enables users to render publication lists from PUMA or BibSonomy in
 *  various styles.
 *
 *  Copyright notice
 *  (c) 2015 Sebastian Böttger <seboettg@gmail.com>
 *
 *  HothoData GmbH (http://www.academic-puma.de)
 *  Knowledge and Data Engineering Group (University of Kassel)
 *
 *  All rights reserved
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\ExtBibsonomyCsl\ViewHelpers;

use AcademicPuma\RestClient\Model\Post;
use \TYPO3\CMS\Fluid\Core\ViewHelper\TagBuilder;

/**
 *
 *
 * @package ext_bibsonomy_csl
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
class CslThumbnailRendererViewHelper extends CslDocumentImageViewHelper
{

    /**
     * @param Post $post
     * @param string $preview
     * @param string $width
     * @param bool $linkDocument
     *
     * @return string
     */
    public function render(Post $post, $preview = "SMALL", $width = "57", $linkDocument = true)
    {

        $documents = $post->getDocuments();
        $fileName  = $documents[0]->getFilename();
        $userName  = $post->getUser()->getName();

        /** @var TagBuilder $img */
        $img = $this->getTagBuilderImage($documents[0], $post->getResource()->getIntraHash(), $userName, $preview,
            $width);

        $arguments = array(
            "intraHash" => $post->getResource()->getIntraHash(),
            "fileName"  => $fileName,
            "userName"  => $userName,
            "preview"   => $preview
        );

        $uriBuilder = $this->controllerContext->getUriBuilder();
        $uriBuilder->reset();
        $arguments['preview'] = "LARGE";
        $large                = $uriBuilder->uriFor("preview", $arguments, "Document", 'extbibsonomycsl',
            'publicationlist');

        $img->addAttribute("onmouseover", "showtrail('" . $large . "');");
        $img->addAttribute("onmouseout", "hidetrail();");

        if ($linkDocument) {
            unset($arguments['preview']);
            $href = $uriBuilder->uriFor("download", $arguments, 'Document', 'extbibsonomycsl', 'publicationlist');
            $a    = new TagBuilder('a');
            $a->addAttribute("href", $href);
            $a->addAttribute("title", "Download " . $fileName);
            $a->addAttribute("target", "_blank");
            $a->setContent($img->render());

            return $a->render();
        } else {
            return '<span>' . $img->render() . '</span>';
        }
    }
}
