<?php

/*
 *  PUMA/BibSonomy CSL (ext_bibsonomy_csl) is a TYPO3 extension which
 *  enables users to render publication lists from PUMA or BibSonomy in
 *  various styles.
 *
 *  Copyright notice
 *  (c) 2015 Sebastian Böttger <seboettg@gmail.com>
 *
 *  HothoData GmbH (http://www.academic-puma.de)
 *  Knowledge and Data Engineering Group (University of Kassel)
 *
 *  All rights reserved
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\ExtBibsonomyCsl\ViewHelpers;

use AcademicPuma\ExtBibsonomyCsl\Lib\Helper;
use AcademicPuma\RestClient\Model\Post;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;

require_once \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath('ext_bibsonomy_csl') . 'vendor/autoload.php';

/**
 *
 *
 * @package ext_bibsonomy_csl
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
class PrintPersonalViewHelper extends AbstractViewHelper
{

    /**
     * @param Post $post
     * @param array $settings
     *
     * @return string
     * @throws \Exception
     */
    public function render(Post $post, array $settings)
    {
        $array = explode("\n", $settings['bib_link_personal']);
        $content = '';
        $authors = explode(' and ', $post->getResource()->getAuthor());
        foreach($authors as $author) {
            foreach($array as $entry) {
                $entry = explode('; ', $entry);
                // Check, if the entry starts with the same lastname and first letter of the firstname as the author
                if (count($entry) === 3 && strpos($entry[0], substr($author, 0, strpos($author, ', ') + 1)) === 0) {
                    $url = (Helper::isUrl('https://' . $entry[1])) ? '<br/> Website: <a href="https://'
                        . $entry[1] . '">' . $entry[1] . '</a>' : '';
                    $email = (filter_var($entry[2], FILTER_VALIDATE_EMAIL)) ? '<br/> E-Mail: <a href="mailto:'
                        . $entry[2] . '">' . $entry[2] . '</a>' : '<br/> E-Mail: ' . $entry[2];
                    $content .= '<p><label>' . $entry[0] . '</label>' . $url . $email . '</p>';
                    break;
                }
            }
        }

        if (!empty($content)) {
            return $this->createModal($authors, $content, $post->getResource()->getInterHash());
        }

        return '';
    }


    /**
     * Generates a HTML modal with bootstrap, which is already used in TYPO3
     * @param $authors  Array   Authors of the publication
     * @param $content  string  Websites and e-mails of the authors involved in this post
     * @param $modalid  string  Unique id for the modal using the interhash of bibsonomy
     * @return          string  HTML modal
     */
    public function createModal($authors, $content, $modalid)
    {
        // Label and title of modal are depending on the amount of authors
        $mode = ".single";
        if (sizeof($authors) > 1) {
            $mode = ".multiple";
        }

        // Link entry for opening the modal
        $button = '<span class="tx-extbibsonomycsl-bib-link">[<a onClick="document.getElementById(\'modal-' . $modalid . '\').style.display=\'inline-block\'" href="#"> '
            . LocalizationUtility::translate("fe.list.link.personal" . $mode, "ext_bibsonomy_csl") . ' </a>]</span>';

        // Modal
        $modal =    '<div id="modal-' . $modalid . '" class="modal">
                        <div class="modal-content">
                            <button class="close" onClick="document.getElementById(\'modal-' . $modalid . '\').style.display=\'none\'" href="#">&times;</button>
                            <h4 class="modal-title">' . LocalizationUtility::translate("fe.list.link.personal.title" . $mode, "ext_bibsonomy_csl") . '</h4>
                            <div class="modal-body">' . $content . '</div>
                        </div>
                    </div>';

        return $button . $modal;
    }
}