<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Model;

/**
 * Description of Post
 *
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
class Post implements ModelObject {

    /**
     * @var \AcademicPuma\RestClient\Model\User
     */
    private $user;

    /**
     * @var \AcademicPuma\RestClient\Model\Groups
     */
    private $group;

    /**
     * @var \AcademicPuma\RestClient\Model\Tags
     */
    private $tag;

    /**
     * @var \AcademicPuma\RestClient\Model\Documents
     */
    private $documents;

    /**
     * @var \AcademicPuma\RestClient\Model\Resource
     */
    private $resource;

    /**
     * @var string
     */
    private $description;

    /**
     * @var string
     */
    private $postingdate;

    /**
     * @var string
     */
    private $changedate;

    /**
     * @return \AcademicPuma\RestClient\Model\User
     */
    public function getUser() {
        return $this->user;
    }

    /**
     * @return \AcademicPuma\RestClient\Model\Groups
     */
    public function getGroup() {
        return $this->group;
    }

    /**
     * @return \AcademicPuma\RestClient\Model\Tags
     */
    public function getTag() {
        return $this->tag;
    }

    /**
     * @return \AcademicPuma\RestClient\Model\Documents
     */
    public function getDocuments() {
        return $this->documents;
    }

    /**
     * @return \AcademicPuma\RestClient\Model\Resource
     */
    public function getResource() {
        return $this->resource;
    }

    /**
     * @return string
     */
    public function getDescription() {
        return $this->description;
    }

    /**
     * @return string
     */
    public function getPostingdate() {
        return $this->postingdate;
    }

    /**
     * @return string
     */
    public function getChangedate() {
        return $this->changedate;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\User $user
     *
     * @return $this
     */
    public function setUser($user) {
        $this->user = $user;
        return $this;
    }


    public function setGroup($group) {
        $this->group = $group;
        return $this;
    }


    public function setTag(array $tag) {
        $this->tag = $tag;
        return $this;
    }
    
    public function addTag($tag) {
        $this->tag[] = $tag;
        return $this;
    }
    
    public function setDocuments($documents) {
        $this->documents = $documents;
        return $this;
    }

    public function setResource($resource) {
        $this->resource = $resource;
        return $this;
    }

    public function setDescription($description) {
        $this->description = $description;
        return $this;
    }

    public function setPostingdate($postingdate) {
        $this->postingdate = $postingdate;
        return $this;
    }

    public function setChangedate($changedate) {
        $this->changedate = $changedate;
        return $this;
    }
    
    public function hasDocuments() {
        return empty($this->documents) ? false : true;
    }
    
    public function hasGroup() {
        return empty($this->group) ? false : true;
    }

    public function __toString() {
        return $this->resource.'/'.$this->user;
    }
}
