<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Model;

/**
 * Represents a tag.
 *
 * @author Florian Fassing
 * @author Sebastian Böttger
 */
class Tag implements ModelObject {

    /**
     * @var array
     */
    private $subTags;

    /**
     * @var array
     */
    private $superTags;

    /**
     * @var string
     */
    private $name;

    /**
     * @var integer
     */
    private $globalcount;

    /**
     * @var integer
     */
    private $usercount;

    /**
     * @var float
     */
    private $score;

    /**
     * @var float
     */
    private $confidence;

    /**
     * @var string
     */
    private $href;

    /**
     *
     * @return array
     */
    public function getSubTags() {
        return $this->subTags;
    }

    public function hasSubTags() {
        return !empty($this->subTags);
    }
    /**
     *
     * @return array
     */
    public function getSuperTags() {
        return $this->superTags;
    }

    public function getName() {
        return $this->name;
    }

    public function getGlobalcount() {
        return $this->globalcount;
    }

    public function getUsercount() {
        return $this->usercount;
    }

    public function getScore() {
        return $this->score;
    }

    public function getConfidence() {
        return $this->confidence;
    }

    public function getHref() {
        return $this->href;
    }

    public function setSubTags(Tags $subTags) {
        $this->subTags = $subTags;
        return $this;
    }

    public function addSubTag(Tag $subTag) {
        $this->subTags[] = $subTag;

        return $this;
    }

    public function setSuperTags(array $superTags) {
        $this->superTags = $superTags;
        return $this;
    }

    public function addSuperTag(Tag $superTag) {
        $this->superTags[] = $superTag;
        return $this;
    }

    public function setName($name) {
        $this->name = $name;
        return $this;
    }

    public function setGlobalcount($globalcount) {
        $this->globalcount = $globalcount;
        return $this;
    }

    public function setUsercount($usercount) {
        $this->usercount = $usercount;
        return $this;
    }

    public function setScore($score) {
        $this->score = $score;
        return $this;
    }

    public function setConfidence($confidence) {
        $this->confidence = $confidence;
        return $this;
    }

    public function setHref($href) {
        $this->href = $href;
        return $this;
    }

    /**
     * @return string
     */
    public function __toString() {
        return $this->name;
    }

    /**
     * @param Tag       $b
     * @param string    $sortingKey
     *
     * @return int
     */
    public function compare(Tag $b, $sortingKey) {

        switch ($sortingKey) {
        case 'name':
            return strcmp($this->getName(), $b->getName());
        case 'usercount':
            return ($this->usercount > $b->getUsercount()) ? 1 : -1;
        case 'globalcount':
            return ($this->globalcount > $b->getGlobalcount()) ? 1 : -1;
        default:
            return 0;
        }

    }
}
