<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Get;

use AcademicPuma\RestClient\Queries\AbstractQuery;
use AcademicPuma\RestClient\Config;
use GuzzleHttp\Client;

/**
 * Builds URL for GetGroupList REST-API calls.
 * Also holds the response after being executed.
 *
 * @author Florian Fassing
 */
class GetGroupListQuery extends AbstractQuery {

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * GET /groups
     *
     * @param int $start    start index
     * @param int $end      end index
     */
    public function __construct($start, $end) {
        parent::__construct();

        // Check wether start and end params make sense.
        if ($start !== NULL && $start < 0) { $start = 0; }
        if ($end !== NULL && $end < $start) { $end = $start; }

        $this->url = $this->urlBuilder->buildUrl(
            [// path
                Config\RESTConfig::GROUPS_URL
            ],
            [// params
                Config\RESTConfig::START_PARAM  => $start,
                Config\RESTConfig::END_PARAM    => $end
            ]
        );
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $this->response = $httpClient->get($this->url, $reqOpts);
        $this->executed = true;

        return $this;
    }
}
