<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Get;

use AcademicPuma\RestClient\Queries\AbstractQuery;
use AcademicPuma\RestClient\Config;
use GuzzleHttp\Client;

/**
 * Query for request a list of Users
 *
 * @since 11.06.15
 * @author Sebastian Böttger / boettger@cs.uni-kassel.de
 */
class GetUserListQuery extends AbstractQuery {


    /**
     * Get bibsonomy's user list.
     *
     * @param int $start
     * @param int $end
     */
    public function __construct($start, $end) {

        parent::__construct();
        if ($start < 0) { $start = 0; }
        if ($end < $start) { $end = $start; }

        $this->url = $this->urlBuilder->buildUrl(
            [Config\RESTConfig::USERS_URL], // path
            [// params
                Config\RESTConfig::START_PARAM => $start,
                Config\RESTConfig::END_PARAM => $end
            ]
        );
    }

    /**
     * executes the query and returns the query
     *
     * @param Client $client
     * @param array  $reqOpts request options
     *
     * @return $this;
     */
    public function execute(Client $client, array $reqOpts = []) {

        $this->response = $client->get($this->url, $reqOpts);
        $this->executed = true;

        return $this;
    }
}