<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Post;

use AcademicPuma\RestClient\Queries\AbstractQuery;
use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Renderer;
use GuzzleHttp\Client;

/**
 * Builds URL for CreateConcept REST-API calls.
 * Also holds the response after being executed.
 *
 * @author Florian Fassing
 */
class CreateConceptQuery extends AbstractQuery {

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * POST /users/[username]/concepts/[conceptname]
     *
     * @param Model\Tag $concept    the concept which will be created
     * @param string    $userName   name of the user who will hold the created concept
     */
    public function __construct(Model\Tag $concept, $userName) {
        parent::__construct();

        $this->url = $this->urlBuilder->buildUrl(
                [// path
                    Config\RESTConfig::USERS_URL,
                    $userName,
                    Config\RESTConfig::CONCEPTS_URL,
                    $concept->getName()
                ],
                [] // no params
        );

        // Render XML from model and write to body.
        $xmlRenderer = new Renderer\XMLModelRenderer();
        $this->body = $xmlRenderer->render($concept);
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $reqOpts['body'] = $this->body;
        $reqOpts['headers'] = ['Content-Type' => 'application/xml'];

        $this->response = $httpClient->post($this->url, $reqOpts);
        $this->executed = true;

        return $this;
    }
}
