<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Post;

use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Renderer;
use AcademicPuma\RestClient\Queries;
use GuzzleHttp\Client;

/**
 * Builds URL for CreatePost REST-API calls.
 * Also holds the response after being executed.
 *
 * @author Florian Fassing
 */
class CreatePostQuery extends Queries\AbstractQuery {

    /**
     * @var string
     */
    private $resourceHash;

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * POST /users/[username]/posts
     *
     * @param Model\Post    $post       the post which will be created
     * @param string        $username   name of the user which will hold the post
     */
    public function __construct(Model\Post $post, $username) {

        parent::__construct();

        $this->url = $this->urlBuilder->buildUrl(
            [// path
                Config\RESTConfig::USERS_URL,
                $username,
                Config\RESTConfig::POSTS_URL
                ],
            [] // no params
        );

        // Render XML from model and write to body.
        $xmlRenderer = new Renderer\XMLModelRenderer();
        $this->body = $xmlRenderer->render($post);
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $reqOpts['body'] = $this->body;
        $reqOpts['headers'] = ['Content-Type' => 'application/xml'];

        $this->response = $httpClient->post($this->url, $reqOpts);

        $this->executed = true;

        // Get resourceHash from response.
        /** @noinspection PhpUndefinedFieldInspection */
        $this->resourceHash = (string) $this->response->xml()->resourcehash;


        return $this;
    }

    /**
     * @return string
     */
    public function getResourceHash() {

        return $this->resourceHash;
    }

    /**
     * @param string $resourceHash
     */
    public function setResourceHash($resourceHash) {

        $this->resourceHash = $resourceHash;
    }
}
