<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Put;

use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Renderer;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Queries;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

/**
 * Builds URL for ChangeConcept REST-API calls.
 * Also holds the response after being executed.
 *
 * @author Florian Fassing
 */
class ChangeConceptQuery extends Queries\AbstractQuery {

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * PUT /users/[username]/concepts/[conceptname]
     *
     * @param Model\Tag $concept    the concept which will be edited
     * @param string    $username   name of the user holding the concept
     * @param string    $operation  TODO: description
     */
    public function __construct(Model\Tag $concept, $username, $operation) {

        parent::__construct();

        $this->url = $this->urlBuilder->buildUrl(
            [// path
                Config\RESTConfig::USERS_URL,
                $username,
                Config\RESTConfig::CONCEPTS_URL,
                $concept->getName()
            ],
            [] // no params
        );

        // Render XML from model and write to body.
        $xmlRenderer = new Renderer\XMLModelRenderer();
        $this->body = $xmlRenderer->render($concept);
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $reqOpts['body'] = $this->body;
        $reqOpts['headers'] = ['Content-Type' => 'application/xml'];

        try {
            $this->response = $httpClient->put($this->url, $reqOpts);
        } catch (RequestException $e) {
            echo $e->getMessage();
        } catch (\LogicException $e) {
            echo $e->getMessage();
        }

        $this->executed = true;
        return $this;
    }
}
