<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Put;

use AcademicPuma\RestClient\Config;
use GuzzleHttp\Client;
use AcademicPuma\RestClient\Queries;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Renderer;

class ChangeDocumentNameQuery extends Queries\AbstractQuery {

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * PUT /users/[username]/posts/[resourcehash]/documents/[fileName]
     *
     * @param string            $userName
     * @param string            $resourceHash the intraHash of the post the document belongs to
     * @param string            $fileName     new name for the file
     * @param Model\Document    $document
     *
     */
    public function __construct($userName, $resourceHash, $fileName, Model\Document $document) {

        parent::__construct();

        $this->url = $this->urlBuilder->buildUrl(
            [// path
                Config\RESTConfig::USERS_URL,
                $userName,
                Config\RESTConfig::POSTS_URL,
                $resourceHash,
                Config\RESTConfig::DOCUMENTS_SUB_PATH,
                $fileName
            ],
            [] // no params
        );

        // Render XML from model and write to body.
        $xmlRenderer = new Renderer\XMLModelRenderer();
        $this->body = $xmlRenderer->render($document);
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $reqOpts['body'] = $this->body;
        $reqOpts['headers'] = ['Content-Type' => 'application/xml'];

        $this->response = $httpClient->put($this->url, $reqOpts);
        $this->executed = true;

        return $this;
    }
}
