<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Renderer;

use AcademicPuma\RestClient\Model;

/**
 * Description of Renderer
 *
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
abstract class ModelRenderer {

    /**
     * Serializes <pre>$posts </pre> recursively into its XML representation
     * @param \AcademicPuma\RestClient\Model\Posts $post
     */
    protected abstract function serializePosts(Model\Posts $post);

    /**
     * Serializes a <pre>$post</pre> recursively into its XML representation
     * @param \AcademicPuma\RestClient\Model\Post $post
     */
    protected abstract function serializePost(Model\Post $post);

    /**
     * Serializes a set of <pre>$tags</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Tags $tags
     */
    protected abstract function serializeTags(Model\Tags $tags);

    /**
     * Serializes a <pre>$tag</pre> into its XML representation
     * @param \AcademicPuma\RestClient\Model\Tag $tag
     */
    protected abstract function serializeTag(Model\Tag $tag);


    /**
     * Serializes a set of <pre>$users</pre> into its XML representation
     * @param \AcademicPuma\RestClient\Model\Users $users
     */
    protected abstract function serializeUsers(Model\Users $users);

    /**
     * Serializes a <pre>$user</pre> into its XML representation
     * @param \AcademicPuma\RestClient\Model\User $user
     */
    protected abstract function serializeUser(Model\User $user);
    
    /**
     * Serializes a <pre>$resource</pre> into its XML representation
     * @param \AcademicPuma\RestClient\Model\Resource $resource
     */
    protected abstract function serializeResource(Model\Resource $resource);

    /**
     * Serializes a set of <pre>$groups</pre> into its XML representation
     * @param \AcademicPuma\RestClient\Model\Groups $groups
     */
    protected abstract function serializeGroups(Model\Groups $groups);

    /**
     * Serializes a <pre>$group</pre> into its XML representation
     * @param \AcademicPuma\RestClient\Model\Group $group
     */
    protected abstract function serializeGroup(Model\Group $group);

    /**
     * Serializes a set of <pre>$documents</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Documents $documents
     */
    protected abstract function serializeDocuments(Model\Documents $documents);

    /**
     * Serializes a <pre>$document</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Document $document
     */
    protected abstract function serializeDocument(Model\Document $document);

    /**
     * Renders a <pre>ModelObject $object</pre> recursively into its representation
     *
     * @param Model\ModelObject $model Object of type <pre>ModelObject</pre>
     *
     * @return null
     */
    public function render(Model\ModelObject $model) {


        if ($model instanceof Model\Posts) {
            return $this->serializePosts($model);
        }

        if ($model instanceof Model\Tags) {
            return $this->serializeTags($model);
        }

        if ($model instanceof Model\Users) {
            return $this->serializeUsers($model);
        }

        if ($model instanceof Model\Groups) {
            return $this->serializeGroups($model);
        }

        if ($model instanceof Model\Documents) {
            return $this->serializeDocuments($model);
        }

        if ($model instanceof Model\Post) {
            return $this->serializePost($model);
        }

        if ($model instanceof Model\Resource) {
            return $this->serializeResource($model);
        }

        if ($model instanceof Model\User) {
            return $this->serializeUser($model);
        }

        if ($model instanceof Model\Tag) {
            return $this->serializeTag($model);
        }

        if ($model instanceof Model\Group) {
            return $this->serializeGroup($model);
        }

        if ($model instanceof Model\Document) {
            return $this->serializeDocument($model);
        }

        return null;
    }
}
