<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Util;

use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Exceptions;

/**
 * Contains helper methods to quickly validate common parameters of RESTClient methods.
 *
 * @author Florian Fassing <florian.fassing@gmail.com>
 */
class ParameterCheck {

    public static function checkResourcetype($resourceType) {

        if ($resourceType === Config\Resourcetype::BOOKMARK || $resourceType === Config\Resourcetype::BIBTEX) {

            return true;

        } else {

            throw new Exceptions\UnsupportedOperationException('Resourcetype ' . $resourceType
                    .' is not available. It has to be ' . Config\Resourcetype::BOOKMARK . ' or '
                    .Config\Resourcetype::BIBTEX . '.');
        }
    }

    public static function checkGrouping($grouping) {

        if ($grouping === Config\Grouping::USER || $grouping === Config\Grouping::GROUPING
                || $grouping === Config\Grouping::GROUP || $grouping == Config\Grouping::VIEWABLE) {

            return true;

        } else {

            throw new Exceptions\UnsupportedOperationException("Grouping '" . $grouping
                    . "' is not available.");
        }
    }

    public static function checkGroupingWithoutGroups($grouping) {

        if ($grouping === Config\Grouping::USER || $grouping === Config\Grouping::ALL) {

            return true;

        } else if ($grouping === Config\Grouping::GROUP) {

            throw new Exceptions\UnsupportedOperationException("Grouping " . Config\Grouping::GROUP
                    . "is not implemented yet");
        } else {

            throw new Exceptions\UnsupportedOperationException("Grouping " . $grouping
                    . " is not available.");
        }
    }

    public static function checkTags($tags) {

        if (!empty($tags) && is_array($tags)) {

            return true;

        } else {

            throw new Exceptions\UnsupportedOperationException("The \$tags param must be filled and"
                    ."has to be an array.");
        }
    }

    public static function checkTagOrder($order) {

        if ($order === Config\TagOrder::FREQUENCY || $order === Config\TagOrder::ALPHANUMERIC) {
            return true;
        }

        throw new Exceptions\UnsupportedOperationException("Tag order '" . $order . "'' is not available.");
    }
}
