<?php

namespace AcademicPuma\RestClient\Queries\Get;

use \AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Config\Sorting;
use \AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Util\Collection\PostComparator;

/**
 * Generated by PHPUnit_SkeletonGenerator on 2015-04-05 at 09:18:09.
 */
class GetPostsQueryTest extends \PHPUnit_Framework_TestCase {

    /**
     * @var GetPostsQuery
     */
    protected $getPostsQuery;

    /**
     * Sets up the fixture, for example, opens a network connection.
     * This method is called before a test is executed.
     */
    protected function setUp() {
    }

    /**
     * Tears down the fixture, for example, closes a network connection.
     * This method is called after a test is executed.
     */
    protected function tearDown() {
    }

    /**
     * @dataProvider executeProvider
     */
    public function testExecute($accessor, $resourceType, $grouping, $groupingName, array $tags, $hash, $search, $format, $start, $end) {

        $this->getPostsQuery = new GetPostsQuery($resourceType, $grouping, $groupingName, $tags, $hash, $search, $format, $start, $end);

        // Check status code.
        $statusCode = $this->getPostsQuery->execute($accessor->getClient())->getStatusCode();
        $this->assertEquals('200', $statusCode);

        // Check content.
        $body = $this->getPostsQuery->getBody();
        $this->assertEquals(preg_match('!<posts start=!', $body), 1);

        // Check type.
        $posts = $this->getPostsQuery->model();

        if ($grouping === Config\Grouping::GROUPING) {
            $this->assertTrue($posts->count() === 40);
        }

        foreach($posts as $post) {
            $this->assertTrue($post->getResource() instanceof Model\Bibtex);
            $this->assertNotEmpty($post->getResource()->getTitle());
        }

        // Is executed flag set?
        $this->assertTrue($this->getPostsQuery->isExecuted());
    }


    public function executeProvider() {
        global $ACCESSOR_OAUTH, $ACCESSOR_BASICAUTH;

        return [
            [$ACCESSOR_BASICAUTH, Config\Resourcetype::BIBTEX, Config\Grouping::USER, TEST_USER_ID, ['myown', 'test'], null, null, 'xml', 0, 0],
            [$ACCESSOR_OAUTH, Config\Resourcetype::BIBTEX, Config\Grouping::GROUPING, Config\Grouping::GROUPING_VALUE_ALL, [], null, null, 'xml', 0, 40],
            [$ACCESSOR_BASICAUTH, Config\Resourcetype::BIBTEX, Config\Grouping::GROUPING, Config\Grouping::GROUPING_VALUE_ALL, [], null, null, 'xml', 0, 40]
        ];
    }

    /**
     * @dataProvider sortingProvider
     */
    public function testSorting($accessor, $resourceType, $grouping, $groupingName, array $tags, $hash, $search, $format, $start, $end) {
        $this->getPostsQuery = new GetPostsQuery($resourceType, $grouping, $groupingName, $tags, $hash, $search, $format, $start, $end);

        $this->getPostsQuery->execute($accessor->getClient());

        /** @var Model\Posts $posts */
        $posts = $this->getPostsQuery->model();

        echo "unordered list:\n";
        foreach ($posts as $post) {
            echo $post->getResource()->getTitle()."\n";
        }
        echo "\n\n";

        $posts->sort('author', Sorting::ORDER_DESC);
        echo "ordered by author (desc):\n";
        foreach ($posts as $post) {
            echo $post->getResource()->getAuthor()."\n";
        }
        echo "\n\n";
        $oldPost = null;
        foreach ($posts as $post) {
            if ($oldPost != null) {
                $this->assertTrue(strcmp($oldPost->getResource()->getAuthor(), $post->getResource()->getAuthor()) >= 0);
            }
            $oldPost = $post;
        }


        $posts->sort('title');
        echo "ordered by title:\n";
        foreach ($posts as $post) {
            echo $post->getResource()->getTitle()."\n";
        }
        echo "\n\n";
        $oldPost = null;
        foreach ($posts as $post) {
            if ($oldPost != null) {
                $this->assertTrue(strcmp($oldPost->getResource()->getTitle(), $post->getResource()->getTitle()) <= 0);
            }
            $oldPost = $post;
        }

        $posts->sort('year', Sorting::ORDER_DESC);
        echo "ordered by year (desc):\n";
        foreach ($posts as $post) {
            echo $post->getResource()->getYear()." ".$post->getResource()->getTitle()."\n";
        }
        $oldPost = null;
        foreach ($posts as $post) {
            if ($oldPost != null) {
                $this->assertTrue($oldPost->getResource()->getYear() >= $post->getResource()->getYear());
            }
            $oldPost = $post;
        }

        $typeOrder = ['presentation','book', 'article', 'inproceedings', 'phdthesis'];
        $posts->sort('entrytype', Sorting::ORDER_ASC, $typeOrder);
        echo "ordered by entrytype\n";
        foreach ($posts as $post) {
            echo $post->getResource()->getEntrytype()."\n";
        }
        $oldPost = null;
        foreach ($posts as $post) {

            if ($oldPost != null) {
                $key_a = array_keys($typeOrder, $oldPost->getResource()->getEntrytype())[0];
                $key_b = array_keys($typeOrder, $post->getResource()->getEntrytype())[0];
                $this->assertTrue($key_a <= $key_b);
            }
            $oldPost = $post;
        }
    }

    public function sortingProvider() {
        global $ACCESSOR_BASICAUTH;

        return [
            [$ACCESSOR_BASICAUTH, Config\Resourcetype::BIBTEX, Config\Grouping::USER, TEST_USER_ID, [], NULL, NULL,
             'xml', 0, 20]
        ];
    }
}
