<?php

/**
 *
 *  Copyright (C) 2006 - 2014 Knowledge & Data Engineering Group,
 *                            University of Kassel, Germany
 *                            http://www.kde.cs.uni-kassel.de/
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

namespace AcademicPuma\RestClient;

use academicpuma\citeproc\CSLUtils;
use AcademicPuma\RestClient\Config\RESTConfig;
use AcademicPuma\RestClient\Config\Resourcetype;
use AcademicPuma\RestClient\Config\Grouping;
use AcademicPuma\RestClient\Config\TestConfig;

use AcademicPuma\RestClient\Accessor\BasicAuthAccessor;
use AcademicPuma\RestClient\Accessor\OAuthAccessor;

use AcademicPuma\RestClient\Renderer\TestHelper;
use AcademicPuma\RestClient\RESTClient;


use AcademicPuma\OAuth\Token\AccessToken;


/**
 * Tests the RESTClient class.
 *
 * @author Florian Fassing
 */
class RESTClientTest extends \PHPUnit_Framework_TestCase {

    /**
     * @var RESTClient
     */
    private $basicAuthClient;

    /**
     * @var RESTClient
     */
    private $oAuthClient;

    /**
     * Create one client for all test methods.
     */
    protected function setUp() {
        // Create http client with basic auth for testing.
        $basicAuthAccessor = new BasicAuthAccessor(BIBSONOMY_HOST_URL, TEST_USER_ID, API_KEY);
        $this->basicAuthClient = new RESTClient($basicAuthAccessor);

        // Create access token and http client with oauth for testing.
        $accessToken = new AccessToken();
        $accessToken->setOauthToken(OAUTH_ACCESS_TOKEN);
        $accessToken->setOauthTokenSecret(OAUTH_ACCESS_TOKEN_SEC);

        //$oAuthAccessor = new OAuthAccessor(TestConfig::URL, $accessToken, TestConfig::CONSUMER_KEY, TestConfig::CONSUMER_SEC);
        //$this->oAuthClient = new RESTClient($oAuthAccessor, 'json');
    }

    /**
     *
     */
    public function testCreateConcept() {
        //Tested in Query
        $this->assertTrue(true);

    }


    public function getConceptDetailsProvider() {
        return [
            ['_foo', TEST_USER_ID]
        ];
    }
    /**
     * @dataProvider getConceptDetailsProvider
     */
    public function testGetConceptDetails($conceptName, $userName) {
        $query = $this->basicAuthClient->getConceptDetails($conceptName, $userName)->getQuery();
        $this->assertEquals('200', $query->getStatusCode());
    }

    public function getConceptsProvider() {
        return [
            ['_foo', TEST_USER_ID]
        ];
    }

    /**
     * @dataProvider getConceptsProvider
     */
    public function testGetConcepts($conceptName, $userName) {

        /** @var Model\Tag $concept */
        $concept = $this->basicAuthClient->getConceptDetails($conceptName, $userName)->model();
        $this->assertNotEmpty($concept);
        $this->assertEquals(200, $this->basicAuthClient->getQuery()->getStatusCode());
        $this->assertContains('_subfoo', $concept->getSubTags());
    }


    public function getDocumentFileProvider() {
        return array(
            array(TEST_USER_ID, 'c49c9ef4dc754980aa36760133790f69', 'Vergleich_Sortierverfahren.pdf',
                  Config\DocumentType::FILE)
        );
    }

    /**
     * @dataProvider getDocumentFileProvider
     *
     * @param $userName
     * @param $resourceHash
     * @param $fileName
     * @param $type
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    public function testGetDocumentFile($userName, $resourceHash, $fileName, $type) {

        $query = $this->basicAuthClient->getDocumentFile($userName, $resourceHash, $fileName, $type)->getQuery();
        $statusCode = $query->getStatusCode();
        $this->basicAuthClient->file()->getContents();
        $this->assertEquals('200', $statusCode);
    }

    /**
     * Data Provider for getGroupsProvider
     * @return array
     */
    public function getGroupsProvider() {
        return array(
            array(0, 5, 5),
            array(1, 5, 4),
            array(3, 5, 2)
        );
    }

    /**
     * @dataProvider getGroupsProvider
     */
    public function testGetGroups($start, $end, $expectedSize) {

        $this->basicAuthClient->getGroups($start, $end);

        /** @var Model\Groups $groups */
        $groups = $this->basicAuthClient->model();
        $this->assertTrue($groups->count() === $expectedSize);
    }


    /* GET POST DETAILS */

    public function getPostDetailsProvider() {
        return [
            [TEST_USER_ID, TEST_RESOURCE_HASH]
        ];
    }

    /**
     * @dataProvider getPostDetailsProvider
     */
    public function testGetPostDetails($userName, $resourceHash) {

        /** @var Model\Post $post */
        $post = $this->basicAuthClient->getPostDetails($userName, $resourceHash)->model();
        $this->assertNotEmpty($post->getResource());

    }

    /* GET POSTS */

    public function getPostsProvider() {
        return [
            [Resourcetype::BIBTEX, Grouping::USER, TEST_USER_ID, ['test'], "", "", 'xml', 0, 0],
            [
                Resourcetype::BIBTEX, Grouping::USER, TEST_USER_ID, [], '271dd1a473eaf0af9840758653746c221', "", 'xml', 0, 0],
            [Resourcetype::BIBTEX, Grouping::USER, TEST_USER_ID, [], "", "", 'xml', 0, 2],
            [Resourcetype::BOOKMARK, Grouping::USER, TEST_USER_ID, [], "", "", 'xml', 0, 0]];
    }
    /**
     * @dataProvider getPostsProvider
     */
    public function testGetPosts($resourceType, $grouping, $groupingName, $tags, $resourceHash, $search, $format, $start, $end) {

        $this->basicAuthClient->getPosts($resourceType, $grouping, $groupingName, $tags, $resourceHash, $search, $format, $start, $end);
        /** @var Model\Posts $posts */
        $posts = $this->basicAuthClient->model();

        $this->assertInstanceOf('\\AcademicPuma\\RestClient\\Model\\Posts', $posts);
        $this->assertNotEmpty($posts->toArray());
        foreach ($posts as $post) {
            $this->assertInstanceOf('\\AcademicPuma\\RestClient\\Model\\Post', $post);
            if ($resourceHash === '271dd1a473eaf0af9840758653746c221') {
                $title = 'The Impact of Resource Title on Tags in Collaborative Tagging Systems';
                /** @var Model\Post $post */
                $this->assertTrue($post->getResource()->getTitle() === $title);
            }
        }

    }

    /* GET TAG DETAILS */

    public function getTagDetailsProvider() {
        return array(
            array('myown')
        );
    }

    /**
     * @dataProvider getTagDetailsProvider
     */
    public function testGetTagDetails($tagName) {
        /** @var Model\Tag $tag */
        $tag = $this->basicAuthClient->getTagDetails($tagName)->model();
        $this->assertEquals('myown', $tag->getName());
    }


    /* GET TAGS */

    public function getTagsProvider() {
        return [
            [Grouping::USER, TEST_USER_ID, 'alph', 0, 20]
        ];
    }
    /**
     * @dataProvider getTagsProvider
     */
    public function testGetTags($grouping, $groupingName, $order, $start, $end) {
        /** @var Model\Tags $tags */
        $tags = $this->basicAuthClient->getTags($grouping, $groupingName, '', $order, $start, $end)->model();
        $this->assertTrue($tags->count() > 0);
    }


    public function testGetTagRelation() {
        $this->basicAuthClient->getTagRelation(Config\Grouping::USER, TEST_USER_ID, Config\TagRelation::RELATED, ['puma'], Config\TagOrder::FREQUENCY, 0, 20);
        $tags = $this->basicAuthClient->model();
        $this->assertNotNull($tags);
        $this->assertTrue($tags->count() > 0);
    }

    public function getUserDetailsProvider() {
        return [[TEST_USER_ID]];
    }

    /**
     * @dataProvider getUserDetailsProvider
     */
    public function testGetUserDetails($userName) {
        $this->basicAuthClient->getUserDetails($userName);
        /** @var Model\User $user */
        $user = $this->basicAuthClient->model();
        $this->assertNotEmpty($user->getName());
    }

    /**
     *
     */
    public function testGetUserRelationship() {
        //Tested in Query
        $this->assertTrue(true);
    }

    /**
     *
     */
    public function testGetUsers() {
        /** @var Model\Users $users */
        $users = $this->basicAuthClient->getUsers(0, 20)->model();
        $this->assertInstanceOf('\\AcademicPuma\\RestClient\\Model\\Users', $users);
        $this->assertNotEmpty($users->toArray());
    }

    /**
     *
     */
    public function testUpdateConcept() {
        //Tested in Query
        $this->assertTrue(true);
    }

    /**
     *
     */
    public function testUpdateGroup() {
        //Tested in Query
        $this->assertTrue(true);
    }

    /**
     *
     */
    public function testUpdatePosts() {
        //Tested in Query
        $this->assertTrue(true);
    }

    /**
     *
     */
    public function testUpdateTags() {
        //Tested in Query
        $this->assertTrue(true);
    }

    /**
     *
     */
    public function testUpdateUser() {
        //Tested in Query
        $this->assertTrue(true);
    }



    public function bibliographyProvider() {
        return [
            ['user', TEST_USER_ID, ['test'], '', '', 0, 100, 'apa', 'en-US']
        ];
    }

    /**
     * @dataProvider bibliographyProvider
     *
     */
    public function testBibliography($grouping, $groupingName, $tags, $hash, $search,  $start, $end, $style, $lang) {


        /** @var TYPE_NAME $style */
        $bib = $this->basicAuthClient->getPosts(Config\Resourcetype::BIBTEX, $grouping, $groupingName, $tags, $hash,
                                                $search, 'xml', $start, $end)
                                     ->bibliography($style, $lang);
        $this->assertTrue(is_string($bib));
        $this->assertStringStartsWith('<ol><li><div class="csl-bib-body">', $bib);
    }

    public function bibtexProvider() {
        return [
            ['user', TEST_USER_ID, [], '', '', 0, 100]
        ];
    }

    /**
     * @dataProvider bibtexProvider
     *
     */
    public function testBibtex($grouping, $groupingName, $tags, $hash, $search,  $start, $end) {


        /** @var string $bibtex */
        $bibtex = $this->basicAuthClient
                 ->getPosts(Config\Resourcetype::BIBTEX, $grouping, $groupingName, $tags, $hash, $search, 'xml', $start, $end)
                 ->bibtex();
        $this->assertTrue(is_string($bibtex));
        $this->assertStringStartsWith('@', $bibtex);

        print_r($bibtex);
    }
}
