<?php

namespace AcademicPuma\RestClient\Renderer;

use AcademicPuma\RestClient\Model;

/**
 * Generated by PHPUnit_SkeletonGenerator on 2015-03-31 at 17:20:50.
 */
class XMLModelRendererTest extends \PHPUnit_Framework_TestCase {

    /**
     * @var \AcademicPuma\RestClient\Renderer\XMLModelRenderer
     */
    protected $xmlRenderer;

    protected $testMap;

    /**
     * Sets up the fixture, for example, opens a network connection.
     * This method is called before a test is executed.
     */
    protected function setUp() {

        $this->xmlRenderer = new XMLModelRenderer();
    }

    /**
     * Tears down the fixture, for example, closes a network connection.
     * This method is called after a test is executed.
     */
    protected function tearDown() {

    }


    public function testSerializePost() {

        $postsObject = TestHelper::loadObject('post.object');
        $postsXML = TestHelper::loadXML('post.xml');

        $actual = $this->xmlRenderer->render($postsObject);
        $this->assertEquals($postsXML, $actual);
    }


    public function testSerializeUser() {

        $postsObject = TestHelper::loadObject('user.object');
        $postsXML = TestHelper::loadXML('user.xml');

        $actual = $this->xmlRenderer->render($postsObject);
        $this->assertEquals($postsXML, $actual);
    }


    public function testSerializeTags() {

        $postsObject = TestHelper::loadObject('tags.object');
        $postsXML = TestHelper::loadXML('tags.xml');

        $actual = $this->xmlRenderer->render($postsObject);
        $this->assertEquals($postsXML, $actual);


        // Concepts
        $superTag = new Model\Tag();
        $superTag->setName('_supertag');

        $subTag = new Model\Tag();
        $subTag->setName('_subtag');

        $superTag->setSubTags(new Model\Tags([$subTag]));

        $tags = new Model\Tags([$superTag]);

        $actualXml = $this->xmlRenderer->render($tags);

        $expectedXml = '
<bibsonomy>
    <tags>
        <tag name="_supertag">
            <subTags>
                <tag name="_subtag"/>
            </subTags>
        </tag>
    </tags>
</bibsonomy>';

        $this->assertXmlStringEqualsXmlString($expectedXml, $actualXml);
    }


    public function testSerializeResource() {

        ####### test bookmark #######
        $postsObject = TestHelper::loadObject('bookmark.object');
        $postsXML = TestHelper::loadXML('bookmark.xml');

        $actual = $this->xmlRenderer->render($postsObject);
        $this->assertEquals($postsXML, $actual);

        ####### test bibtex #######
        $postsObject = TestHelper::loadObject('bibtex.object');
        $postsXML = TestHelper::loadXML('bibtex.xml');

        $actual = $this->xmlRenderer->render($postsObject);
        $this->assertEquals($postsXML, $actual);
    }


    public function testSerializePosts() {

        $postsObject = TestHelper::loadObject('posts.object');
        $postsXML = TestHelper::loadXML('posts.xml');

        //print_r($postsObject);
        $actual = $this->xmlRenderer->render($postsObject);
        $this->assertEquals($postsXML, $actual);
    }


    public function testSerializeGroups() {

        $group = TestHelper::loadObject('group.object');

        $groups = new Model\Groups();

        $groups[0] = $group;
        $groups[1] = $group;

        $actual = $this->xmlRenderer->render($groups);
        $expected = '<bibsonomy><groups><group name="seminar" href="http://host.tld/api/groups/seminar"/><group name="seminar" href="http://host.tld/api/groups/seminar"/></groups></bibsonomy>';
        $this->assertXmlStringEqualsXmlString($expected, $actual);
    }


    public function testSerializeDocuments() {
        // Create document.
        $document = new Model\Document();
        $document->setFilename('foo.bar');
        $document->setUserName(TEST_USER_ID);

        $documents = new Model\Documents();
        $documents[0] = $document;
        $documents[1] = $document;

        $actual = $this->xmlRenderer->render($documents);
        $expected = '<bibsonomy><documents><document filename="foo.bar"/><document filename="foo.bar"/></documents></bibsonomy>';

        $this->assertXmlStringEqualsXmlString($expected, $actual);
    }

    public function testSerializeUsers() {
        $user = new Model\User();
        $user->setName('mustermann');
        $user->setRealname('Max Mustermann');
        $user->setEmail('max.mustermann@host.tld');

        $users = new Model\Users();
        $users[0] = $user;
        $users[1] = $user;

        $expected = '<bibsonomy><users><user name="mustermann" realname="Max Mustermann" email="max.mustermann@host.tld"/><user name="mustermann" realname="Max Mustermann" email="max.mustermann@host.tld"/></users></bibsonomy>';
        $actual = $this->xmlRenderer->render($users);

        $this->assertXmlStringEqualsXmlString($expected, $actual);

    }
}